@@ -8,7 +8,7 @@ E.g., for "Mel Gibson" the referred pages would be:
     biography:      http://akas.imdb.com/name/nm0000154/bio
     ...and so on...
 
-Copyright 2004-2006 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2007 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,12 +27,12 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from imdb.Movie import Movie
 from imdb.utils import analyze_name, build_name, canonicalName, \
-                        normalizeName, analyze_title
-from utils import ParserBase
+                        normalizeName, analyze_title, date_and_notes
+from utils import ParserBase, build_movie
 
 
 class HTMLMaindetailsParser(ParserBase):
-    """Parser for the "categorized" page of a given person.
+    """Parser for the "categorized" (maindetails) page of a given person.
     The page should be provided as a string, as taken from
     the akas.imdb.com server.  The final result will be a
     dictionary, with a key for every relevant section.
@@ -42,272 +42,209 @@ class HTMLMaindetailsParser(ParserBase):
         result = cparser.parse(categorized_html_string)
     """
 
-    # Do not gather names and titles references.
-    getRefs = 0
-
-    def _init(self):
-        # This is the dictionary that will be returned by the parse() method.
-        self._person_data = {}
-
     def _reset(self):
-        """Reset the parser."""
-        self._person_data.clear()
-        self._in_name = 0
-        self._name = u''
-        self._in_birth = 0
-        self._in_death = 0
-        self._birth = ''
-        self._death = ''
-        self._in_list = 0
-        self._in_title = 0
-        self._title = ''
-        self._seen_br = 0
-        self._roles = ''
-        self._last_imdbID = ''
-        self._get_imdbID = 0
-        self._in_sect = 0
-        self._in_b = 0
-        self._in_i = 0
-        self._sect_name = ''
-        self._in_emailfriend = 0
-        self._in_akas = 0
-        self._aka = u''
-        self._akas = []
+        self._data = {}
+        self._in_title = False
+        self._title = u''
+        # Get section names.
+        self._in_h5 = False
+        self._section = u''
+        # Sections are ended by br tags.
+        self._in_post_section = False
+        # Stop before the Additional Details section.
+        self._stop_here = False
+        self._in_tn15more = False
+        # Most of the information are stored here.
+        self._cur_txt = u''
+        # Handle filmography.
+        self._in_ol = False
+        self._in_i = False
+        self._in_movie = False
+        # The movie title/role/notes is stored here.
+        self._movie = u''
+        # Stop before akas/episodes information.
+        self._seen_br = False
+        self._in_headshot = False
         self._cur_status = u''
+        # Get a movieID.
+        self._last_imdbID = None
+        self._get_imdbID = False
 
     def get_data(self):
-        """Return the dictionary."""
-        # Split birth/death date/notes.
-        b = self._birth.split('::')
-        if b:
-            b_date = b[0]
-            del b[0]
-            b_notes = ''.join(b)
-            if b_date:
-                self._person_data['birth date'] = b_date.strip()
-            if b_notes:
-                self._person_data['birth notes'] = b_notes.strip()
-        d = self._death.split('::')
-        if d:
-            d_date = d[0]
-            del d[0]
-            d_notes = ''.join(d)
-            if d_date:
-                self._person_data['death date'] = d_date.strip()
-            if d_notes:
-                self._person_data['death notes'] = d_notes.strip()
-        if self._akas:
-            self._person_data['akas'] = self._akas
-        if self._person_data.has_key('miscellaneouscrew'):
-            self._person_data['miscellaneous crew'] = \
-                    self._person_data['miscellaneouscrew']
-            del self._person_data['miscellaneouscrew']
-        return self._person_data
+        return self._data
 
     def start_title(self, attrs):
-        self._in_name = 1
+        self._in_title = True
 
     def end_title(self):
-        self._in_name = 0
-        d = analyze_name(self._name.strip(), canonical=1)
-        self._person_data.update(d)
+        self._in_title = False
+        self._title = self._title.strip()
+        if self._title:
+            self._data.update(analyze_name(self._title, canonical=1))
+
+    def start_h5(self, attrs):
+        if self._stop_here or not self._in_content: return
+        self._seen_br = False
+        self._in_h5 = True
+        self._in_post_section = False
+        self._section = u''
+        self._cur_txt = u''
+        self._in_movie = False
+
+    def end_h5(self):
+        if self._stop_here or not self._in_content: return
+        self._in_h5 = False
+        self._section = str(self._section.strip().lower())
+        if self._section[-1:] == ':':
+            self._section = self._section[:-1].rstrip()
+        # XXX: I don't like this way at all: here we're excluding some
+        #      useless section, but who knows how many other there can be?
+        if self._section in (u'', 'genres', 'awards', 'publicity listings',
+                            'other works', 'trivia'):
+            return
+        # Do some basic transformation.
+        if self._section in ('sometimes credited as', 'alternate names'):
+            self._section = 'akas'
+        elif self._section == 'date of birth':
+            self._section = 'birth date'
+        elif self._section == 'date of death':
+            self._section = 'death date'
+        self._in_post_section = True
 
     def do_img(self, attrs):
-        alt = self.get_attr_value(attrs, 'alt')
-        if alt and alt.lower().strip() == \
-                build_name(self._person_data).lower():
+        if self._in_headshot:
             src = self.get_attr_value(attrs, 'src')
-            if src: self._person_data['headshot'] = src
-        if self._in_list:
-            self._in_list = 0
-
-    def do_br(self, attrs):
-        self._seen_br = 1
-        # Birth/death date/notes are separated by a <br> tag.
-        if self._in_birth and self._birth:
-            self._birth += '::'
-        elif self._in_death and self._death:
-            self._death += '::'
-        elif self._in_list:
-            self._in_list = 0
-        elif self._in_akas:
-            self._aka = self._aka.strip()
-            if self._aka:
-                self._akas.append(self._aka)
-                self._aka = ''
-
-    def start_b(self, attrs):
-        self._in_b = 1
-
-    def end_b(self):
-        self._in_b = 0
-
-    def start_p(self, attrs):
-        cl = self.get_attr_value(attrs, 'class')
-        if cl == 'ch':
-            self._in_b = 1
-
-    def end_p(self): pass
-
-    def start_form(self, attrs):
-        act = self.get_attr_value(attrs, 'action')
-        if act and act.lower() == '/emailafriend':
-            self._in_emailfriend = 1
-
-    def end_form(self):
-        self._in_emailfriend = 0
+            if src:
+                self._data['headshot'] = src
+        if self._stop_here or not self._in_content: return
+        if self.get_attr_value(attrs, 'alt') == 'Additional Details':
+            self._stop_here = True
 
     def do_input(self, attrs):
-        if self._in_emailfriend and not self._person_data.has_key('name'):
-            name = self.get_attr_value(attrs, 'name')
-            if name and name.lower() == 'arg':
-                value = self.get_attr_value(attrs, 'value')
-                if value:
-                    d = analyze_name(value, canonical=1)
-                    self._person_data.update(d)
+        itype = self.get_attr_value(attrs, 'type')
+        if itype is None or itype.lower() != 'hidden': return
+        iname = self.get_attr_value(attrs, 'name')
+        if iname is None or iname != 'primary': return
+        ivalue = self.get_attr_value(attrs, 'value')
+        if ivalue is None: return
+        # It's hard to catch the correct 'Surname, Name' from the
+        # title, so if the "credited alongside another name" form
+        # is found, use it.
+        self._data.update(analyze_name(ivalue, canonical=0))
 
-    def start_ol(self, attrs): pass
+    def start_div(self, attrs): pass
 
-    def end_ol(self):
-        self._death += '::'
-        self._in_sect = 0
-        self._sect_name = ''
+    def end_div(self):
+        pass
+        self.do_br([])
 
-    def start_i(self, attrs):
-        self._in_i = 1
-
-    def end_i(self):
-        self._in_i = 0
-
-    def start_li(self, attrs):
-        self._in_list = 1
-        self._seen_br = 0
-        self._get_imdbID = 1
-
-    def end_li(self):
-        if self._title and self._sect_name:
-            tit = self._title
-            notes = ''
-            self._roles = self._roles.strip()
-            if len(self._roles) > 5:
-                if self._roles[0] == '(' and self._roles[1:5].isdigit():
-                    endp = self._roles.find(')')
-                    if endp != -1:
-                        tit += ' %s' % self._roles[:endp+1]
-                        self._roles = self._roles[endp+1:].strip()
-            if self._roles.startswith('(TV)'):
-                tit += ' (TV)'
-                self._roles = self._roles[4:].strip()
-            elif self._roles.startswith('TV Series'):
-                self._roles = self._roles[10:].strip()
-            elif self._roles.startswith('(V)'):
-                tit += ' (V)'
-                self._roles = self._roles[3:].strip()
-            elif self._roles.startswith('(mini)'):
-                tit += ' (mini)'
-                self._roles = self._roles[6:].strip()
-                if self._roles.startswith('TV Series'):
-                    self._roles = self._roles[10:].strip()
-            elif self._roles.startswith('(VG)'):
-                tit += ' (VG)'
-                self._roles = self._roles[4:].strip()
-            has_note = 0
-            if self._roles.find('(') != -1: has_note = 1
-            sp = self._roles.split('....')
-            if len(sp) == 2:
-                notes = sp[0].strip().replace('  ', ' ')
-                self._roles = sp[1].strip()
-                fn = self._roles.find('(')
-                if fn != -1:
-                    en = self._roles.rfind(')')
-                    pnote = self._roles[fn:en+1].strip()
-                    self._roles = '%s %s' % (self._roles[:fn].strip(),
-                                            self._roles[en+1:].strip())
-                    self._roles = self._roles.strip()
-                    if pnote:
-                        if notes: notes += ' '
-                        notes += pnote
-            else:
-                notes = ''.join(sp).strip().replace('  ', ' ')
-                self._roles = ''
-            movie = Movie(movieID=str(self._last_imdbID), title=tit,
-                            accessSystem='http')
-            if notes: movie.notes = notes
-            movie.currentRole = self._roles
-            self._cur_status = self._cur_status.strip()
-            if self._cur_status:
-                if self._cur_status[0] == '(':
-                    self._cur_status = self._cur_status[1:]
-                if self._cur_status[-1] == ')':
-                    self._cur_status = self._cur_status[:-1]
-                movie['status'] = '%s' % self._cur_status
-                self._cur_status = u''
-            sect = self._sect_name.strip().lower()
-            self._person_data.setdefault(sect, []).append(movie)
-        self._title = ''
-        self._roles = ''
-        self._in_list = 0
-        self._seen_br = 0
-
-    def start_dd(self, attrs): pass
-
-    def end_dd(self):
-        self._in_birth = 0
-        self._in_death = 0
-        if self._in_akas:
-            self._aka = self._aka.strip()
-            if self._aka:
-                self._akas.append(self._aka)
-                self._aka = ''
-            self._in_akas = 0
+    def do_br(self, attrs):
+        if self._stop_here or not self._in_content: return
+        # Inside li tags in filmography, some useless information after a br.
+        self._seen_br = True
+        self._cur_txt = self._cur_txt.strip()
+        if not (self._in_post_section and self._section and self._cur_txt):
+            self._in_post_section = False
+            self._cur_txt = u''
+            return
+        # We're at the end of a section.
+        if self._section == 'birth date':
+            date, notes = date_and_notes(self._cur_txt)
+            if date:
+                self._data['birth date'] = date
+            if notes:
+                self._data['birth notes'] = notes
+        elif self._section == 'death date':
+            date, notes = date_and_notes(self._cur_txt)
+            if date:
+                self._data['death date'] = date
+            if notes:
+                self._data['death notes'] = notes
+        elif self._section == 'akas':
+            akas = self._cur_txt.split(' / ')
+            if akas: self._data['akas'] = akas
+        # XXX: not providing an 'else', we're deliberately ignoring
+        #      other sections.
+        self._in_post_section = False
+        self._cur_txt = u''
 
     def start_a(self, attrs):
+        name = self.get_attr_value(attrs, 'name')
+        if name and name.lower() == 'headshot':
+            self._in_headshot = True
+        if self._stop_here or not self._in_content: return
+        cls = self.get_attr_value(attrs, 'class')
+        # Detect "more" links.
+        if cls and cls.startswith('tn15more'):
+            self._in_tn15more = True
+        if not (self._in_movie and self._get_imdbID): return
         href = self.get_attr_value(attrs, 'href')
         # A movie title.
-        if self._get_imdbID and href and href.find('/title/tt') != -1:
-            self._in_title = 1
+        if href and href.find('/title/tt') != -1:
             imdbID = self.re_imdbID.findall(href)
             if imdbID:
                 self._last_imdbID = imdbID[-1]
-                self._get_imdbID = 0
-        elif self._in_b:
-            name = self.get_attr_value(attrs, 'name')
-            if name: self._in_sect = 1
+                self._get_imdbID = False
 
     def end_a(self):
-        self._in_title = 0
-        self._in_sect = 0
+        if self._in_headshot:
+            self._in_headshot = False
+        if self._in_tn15more:
+            self._in_tn15more = False
+
+    def start_ol(self, attrs):
+        if self._stop_here or not self._in_content: return
+        # We're not in a informational section, but in a filmography list.
+        self._in_post_section = False
+        self._in_ol = True
+        self._cur_txt = u''
+
+    def end_ol(self):
+        if self._stop_here or not self._in_content: return
+        self._in_ol = False
+
+    def start_li(self, attrs):
+        if self._stop_here or not self._in_content: return
+        # We're reading a movie title/role/notes.
+        if self._in_ol:
+            self._get_imdbID = True
+            self._in_movie = True
+        self._last_imdbID = None
+        self._cur_status = u''
+        self._movie = u''
+        self._seen_br = False
+
+    def end_li(self):
+        if self._stop_here or not self._in_content: return
+        self._get_imdbID = False
+        if not self._in_movie: return
+        self._movie = self._movie.strip()
+        self._cur_status = self._cur_status.strip()
+        if not (self._movie and self._last_imdbID and self._section): return
+        # Add this movie to the list.
+        movie = build_movie(self._movie, movieID=self._last_imdbID,
+                status=self._cur_status)
+        self._data.setdefault(self._section, []).append(movie)
+
+    def start_i(self, attrs):
+        self._in_i = True
+
+    def end_i(self):
+        self._in_i = False
 
     def _handle_data(self, data):
-        sdata = data.strip()
-        sldata = sdata.lower()
-        if self._in_name:
-            self._name += data
-        elif self._in_birth:
-            if self._birth and not self._birth[-1].isspace():
-                self._birth += ' '
-            self._birth += sdata
-        elif self._in_death:
-            if self._death and not self._death[-1].isspace():
-                self._death += ' '
-            self._death += sdata
-        elif self._in_akas:
-            self._aka += data
-        elif sldata.startswith('date of death'):
-            self._in_death = 1
-        elif sldata.startswith('date of birth'):
-            self._in_birth = 1
-        elif sldata.startswith('sometimes credited as'):
-            self._in_akas = 1
-        elif self._in_sect:
-            self._sect_name += sldata
-        elif self._in_title and not self._seen_br:
+        if self._in_title:
             self._title += data
-        elif self._in_list:
+        if self._stop_here or not self._in_content: return
+        if self._in_h5:
+            self._section += data
+        elif self._in_post_section and not self._in_tn15more:
+            self._cur_txt += data
+        elif self._in_movie and not self._seen_br:
             if self._in_i:
                 self._cur_status += data
             else:
-                self._roles += data
+                self._movie += data
 
 
 class HTMLBioParser(ParserBase):
@@ -320,6 +257,9 @@ class HTMLBioParser(ParserBase):
         bioparser = HTMLBioParser()
         result = bioparser.parse(biography_html_string)
     """
+    # XXX: this parser is really old; consider a rewrite.
+    _defGetRefs = True
+
     def _init(self):
         # This is the dictionary that will be returned by the parse() method.
         self._bio_data = {}
@@ -327,8 +267,8 @@ class HTMLBioParser(ParserBase):
     def _reset(self):
         """Reset the parser."""
         self._bio_data.clear()
-        self._sect_name = ''
-        self._sect_data = ''
+        self._sect_name = u''
+        self._sect_data = u''
         self._in_sect = 0
         self._in_sect_name = 0
 
@@ -336,7 +276,20 @@ class HTMLBioParser(ParserBase):
         """Return the dictionary."""
         return self._bio_data
 
+    def _end_content(self):
+        self._add_items()
+        if self._bio_data.has_key('mini biography'):
+            nl = []
+            for bio in self._bio_data['mini biography']:
+                byidx = bio.rfind('IMDb Mini Biography By')
+                if byidx != -1:
+                    bio = u'%s::%s' % (bio[byidx+23:].lstrip(),
+                                        bio[:byidx].rstrip())
+                nl.append(bio)
+            self._bio_data['mini biography'][:] = nl
+
     def do_br(self, attrs):
+        self._add_items()
         if self._sect_name.lower() == 'mini biography' and self._sect_data \
                 and  not self._sect_data[-1].isspace():
             self._sect_data += ' '
@@ -345,29 +298,45 @@ class HTMLBioParser(ParserBase):
 
     def end_a(self): pass
 
-    def start_dt(self, attrs):
-        self._in_sect = 1
-        self._in_sect_name = 1
+    def start_h5(self, attrs):
+        if self._in_content:
+            self._add_items()
+            self._in_sect = 1
+            self._in_sect_name = 1
+            self._sect_name = u''
 
-    def end_dt(self):
-        self._in_sect_name = 0
+    def end_h5(self):
+        if self._in_sect_name:
+            self._in_sect_name = 0
+
+    def do_hr(self, attrs):
+        if self._in_content: self._in_content = 0
 
     def start_dd(self, attrs): pass
 
-    def end_dd(self):
+    def _add_items(self):
         # Add a new section in the biography.
-        if self._sect_name and self._sect_data:
+        if self._in_content and self._sect_name and self._sect_data:
             sect = self._sect_name.strip().lower()
-            # XXX: to get rid of the last colons.
+            # XXX: to get rid of the last colons and normalize section names.
             if sect[-1] == ':':
                 sect = sect[:-1]
-            if sect == 'salary': sect = 'salary history'
-            elif sect == 'nickname': sect = 'nick names'
-            elif sect == 'where are they now': sect = 'where now'
-            elif sect == 'personal quotes': sect = 'quotes'
+            if sect == 'salary':
+                sect = 'salary history'
+            elif sect == 'nickname':
+                sect = 'nick names'
+            elif sect == 'where are they now':
+                sect = 'where now'
+            elif sect == 'personal quotes':
+                sect = 'quotes'
+            elif sect == 'date of birth':
+                sect = 'birth date'
+            elif sect == 'date of death':
+                sect = 'death date'
             data = self._sect_data.strip()
             d_split = data.split('::')
             d_split[:] = filter(None, [x.strip() for x in d_split])
+            # Do some transformation on some special cases.
             if sect == 'salary history':
                 newdata = []
                 for j in d_split:
@@ -376,26 +345,40 @@ class HTMLBioParser(ParserBase):
                 d_split[:] = newdata
             elif sect == 'nick names':
                 d_split[:] = [normalizeName(x) for x in d_split]
-            if sect == 'birth name':
-                self._bio_data[sect] = canonicalName(d_split[0])
+            elif sect == 'birth name':
+                d_split = canonicalName(d_split[0])
             elif sect == 'height':
-                self._bio_data[sect] = d_split[0]
-            elif sect == 'imdb mini-biography by' and \
-                    self._bio_data.has_key('mini biography'):
-                self._bio_data['mini biography'][-1] = '%s::%s' % (d_split[0],
-                                    self._bio_data['mini biography'][-1])
-
+                d_split = d_split[0]
+            elif sect == 'spouse':
+                d_split[:] = [x.replace(' (', '::(', 1).replace(' ::', '::')
+                                for x in d_split]
+            # Birth/death date are in both maindetails and bio pages;
+            # it's safe to collect both of them.
+            if sect == 'birth date':
+                date, notes = date_and_notes(d_split[0])
+                if date:
+                    self._bio_data['birth date'] = date
+                if notes:
+                    self._bio_data['birth notes'] = notes
+            elif sect == 'death date':
+                date, notes = date_and_notes(d_split[0])
+                if date:
+                    self._bio_data['death date'] = date
+                if notes:
+                    self._bio_data['death notes'] = notes
             elif d_split:
                 # Multiple items are added separately (e.g.: 'trivia' is
                 # a list of strings).
                 self._bio_data[sect] = d_split
-        self._sect_name = ''
-        self._sect_data = ''
+        self._sect_name = u''
+        self._sect_data = u''
         self._in_sect = 0
 
     def start_p(self, attrs):
         if self._in_sect:
             if self._sect_data:
+                if self._sect_data[-1].isspace():
+                    self._sect_data = self._sect_data.rstrip()
                 self._sect_data += '::'
 
     def end_p(self): pass
@@ -404,7 +387,7 @@ class HTMLBioParser(ParserBase):
         if self._in_sect:
             if self._sect_data:
                 if self._sect_data[-1].isspace():
-                    self._sect_data = self._sect_data.strip()
+                    self._sect_data = self._sect_data.rstrip()
                 self._sect_data += '::'
 
     def end_tr(self): pass
@@ -421,9 +404,6 @@ class HTMLBioParser(ParserBase):
         if self._in_sect_name:
             self._sect_name += data
         elif self._in_sect:
-            if not data.isspace() and self._sect_data \
-                    and self._sect_data[-1].isspace():
-                data = data.strip()
             self._sect_data += data.replace('\n', ' ')
 
 
@@ -437,15 +417,15 @@ class HTMLOtherWorksParser(ParserBase):
         owparser = HTMLOtherWorksParser()
         result = owparser.parse(otherworks_html_string)
     """
+    _defGetRefs = True
 
     def _init(self):
         self.kind = 'other works'
 
     def _reset(self):
         """Reset the parser."""
-        self._in_ow = 0
         self._ow = []
-        self._cow = ''
+        self._cow = u''
         self._dostrip = 0
 
     def get_data(self):
@@ -461,23 +441,17 @@ class HTMLOtherWorksParser(ParserBase):
     def start_b(self, attrs): pass
 
     def end_b(self):
-        if self.kind == 'agent' and self._in_ow and self._cow:
+        if self.kind == 'agent' and self._in_content and self._cow:
             self._cow += '::'
             self._dostrip = 1
 
     def do_br(self, attrs):
-        if self._in_ow and self._cow:
+        if self._in_content and self._cow:
             self._ow.append(self._cow.strip())
-            self._cow = ''
-
-    def start_dl(self, attrs): pass
-
-    def end_dl(self):
-        self.do_br([])
-        self._in_ow = 0
+            self._cow = u''
 
     def _handle_data(self, data):
-        if self._in_ow:
+        if self._in_content:
             if self._dostrip:
                 data = data.lstrip()
                 if data: self._dostrip = 0
@@ -494,15 +468,10 @@ class HTMLSeriesParser(ParserBase):
         sparser = HTMLSeriesParser()
         result = sparser.parse(filmoseries_html_string)
     """
-
-    # Do not gather names and titles references.
-    getRefs = 0
-
     def _reset(self):
         """Reset the parser."""
         self._episodes = {}
         self._seen_h1 = 0
-        self._in_b = 0
         self._in_episodes = 0
         self._in_ol = 0
         self._in_li = 0
@@ -517,24 +486,29 @@ class HTMLSeriesParser(ParserBase):
         self._misc_info = u''
         self._in_i = 0
         self._got_i_info = 0
+        self._in_span = 0
 
     def get_data(self):
         """Return the dictionary."""
         if not self._episodes: return {}
         return {'episodes': self._episodes}
 
-    def start_h1(self, attrs): pass
+    def start_h1(self, attrs): self._seen_h1 = 1
 
-    def end_h1(self): self._seen_h1 = 1
-
-    def start_b(self, attrs): self._in_b = 1
-
-    def end_b(self): self._in_b = 0
+    def end_h1(self): self._seen_h1 = 0
 
     def start_i(self, attrs): self._in_i = 1
 
     def end_i(self): self._in_i = 0
 
+    def start_div(self, attrs):
+        if not self._in_content: return
+        if self.get_attr_value(attrs, 'class') == 'filmo':
+            self._in_episodes = 1
+
+    def end_div(self):
+        if self._in_episodes: self._in_episodes = 0
+
     def start_ol(self, attrs): self._in_ol = 1
 
     def end_ol(self):
@@ -554,7 +528,7 @@ class HTMLSeriesParser(ParserBase):
             minfo = self._misc_info.strip()
             if et and self._episode_id:
                 eps_data = analyze_title(et, canonical=1)
-                eps_data['kind'] = 'episode'
+                eps_data['kind'] = u'episode'
                 e = Movie(movieID=str(self._episode_id), data=eps_data,
                             accessSystem='http')
                 e['episode of'] = self._cur_series
@@ -571,10 +545,10 @@ class HTMLSeriesParser(ParserBase):
                 rolei = minfo.find(' - ')
                 if rolei != -1:
                     if not self._got_i_info:
-                        role = ''
+                        role = u''
                         role = minfo[rolei+3:].strip()
                         notei = role.rfind('(')
-                        note = ''
+                        note = u''
                         if notei != -1 and role and role[-1] == ')':
                             note = role[notei:]
                             role = role[:notei].strip()
@@ -591,6 +565,12 @@ class HTMLSeriesParser(ParserBase):
         self._in_misc_info = 0
         self._misc_info = u''
 
+    def start_span(self, attrs):
+        self._in_span = 1
+
+    def end_span(self):
+        self._in_span = 0
+
     def start_a(self, attrs):
         if not self._in_episodes: return
         if self._in_ol:
@@ -602,7 +582,7 @@ class HTMLSeriesParser(ParserBase):
             self._in_episode_title = 1
             self._episode_id = mid[0]
             return
-        else:
+        elif self._in_span:
             href = self.get_attr_value(attrs, 'href')
             if not href: return
             mid = self.re_imdbID.findall(href)
@@ -624,12 +604,7 @@ class HTMLSeriesParser(ParserBase):
                 self._cur_series = s
 
     def _handle_data(self, data):
-        if self._seen_h1 and self._in_b:
-            if data.strip().lower().find('filmography by series'):
-                self._seen_h1 = 0
-                self._in_episodes = 1
-                return
-        elif self._in_episode_title:
+        if self._in_episode_title:
             self._cur_episode_title += data
         elif self._in_series_title:
             self._cur_series_title += data
@@ -642,6 +617,92 @@ class HTMLSeriesParser(ParserBase):
             self._misc_info += data
 
 
+class HTMLPersonGenresParser(ParserBase):
+    """Parser for the "by genre" and "by keywords" pages of a given person.
+    The page should be provided as a string, as taken from
+    the akas.imdb.com server.  The final result will be a
+    dictionary, with a key for every relevant section.
+
+    Example:
+        gparser = HTMLPersonGenresParser()
+        result = gparser.parse(bygenre_html_string)
+    """
+    kind = 'genres'
+
+    def _reset(self):
+        """Reset the parser."""
+        self._info = {}
+        self._cur_key = ''
+        self._cur_title = u''
+        self._in_table = False
+        self._in_li = False
+        self._cur_movieID = None
+
+    def get_data(self):
+        """Return the dictionary."""
+        if not self._info: return {}
+        return {self.kind: self._info}
+
+    def start_table(self, attrs):
+        if not self._in_content: return
+        self._in_table = True
+
+    def end_table(self):
+        self._in_table = False
+
+    def start_a(self, attrs):
+        if not self._in_content: return
+        if self._in_li:
+            href = self.get_attr_value(attrs, 'href')
+            if href:
+                imdbID = self.re_imdbID.findall(href)
+                if imdbID:
+                    self._cur_movieID = imdbID[-1]
+                    return
+        if not self._in_table: return
+        name = self.get_attr_value(attrs, 'name')
+        if name:
+            self._cur_key = name
+
+    def end_a(self): pass
+
+    def start_li(self, attrs):
+        if not (self._in_content and self._cur_key): return
+        self._in_li = True
+
+    def end_li(self):
+        self._in_li = False
+        self._add_info()
+
+    def do_br(self, attrs):
+        if not self._in_li: return
+        self._in_li = False
+        self._add_info()
+
+    def _add_info(self):
+        self._cur_key = self._cur_key.strip()
+        self._cur_title = self._cur_title.strip()
+        if not (self._cur_key and self._cur_title and self._cur_movieID):
+            self._cur_title = u''
+            self._cur_movieID = None
+            return
+        ridx = self._cur_title.find('[')
+        notes = u''
+        if ridx != -1:
+            notes = self._cur_title[ridx:].lstrip()
+            self._cur_title = self._cur_title[:ridx].rstrip()
+        m = build_movie(self._cur_title, movieID=self._cur_movieID)
+        m.notes = notes
+        self._info.setdefault(self._cur_key.replace('X2D', '-'), []).append(m)
+        self._cur_title = u''
+        self._cur_movieID = None
+
+    def _handle_data(self, data):
+        if not self._in_li: return
+        self._cur_title += data
+
+
+
 # The used instances.
 maindetails_parser = HTMLMaindetailsParser()
 bio_parser = HTMLBioParser()
@@ -658,4 +719,11 @@ publicity_parser = HTMLTechParser()
 publicity_parser.kind = 'publicity'
 from movieParser import news_parser
 person_series_parser = HTMLSeriesParser()
+from movieParser import HTMLLocationsParser
+person_contacts_parser = HTMLTechParser()
+person_contacts_parser.kind = 'contacts'
+from movieParser import sales_parser
+person_genres_parser = HTMLPersonGenresParser()
+person_keywords_parser = HTMLPersonGenresParser()
+person_keywords_parser.kind = 'keywords'
 
